# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# xoutil.progress
# ---------------------------------------------------------------------
# Copyright (c) 2013-2015 Merchise Autrement and Contributors
# Copyright (c) 2011, 2012 Medardo Rodríguez
# All rights reserved.
#
# Author: Medardo Rodriguez
# Contributors: see CONTRIBUTORS and HISTORY file
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.

'''Tool to show a progress percent in the terminal.'''


from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode)

from xoutil.names import strlist as strs
__all__ = strs('Progress')
del strs


_HELIX = '|/-\\'


class Progress(object):
    '''
    Print a progress percent to the console. Also the elapsed and the
    estimated times.

    To signal an increment in progress just call the instance and (optionally)
    pass a message like in::

        progress = Progress(10)
        for i in range(10):
            progress()
    '''

    def __init__(self, max_value=100, delta=1, first_message=None,
                 display_width=None):
        from xoutil.datetime import datetime
        self.max_value = max_value
        self.delta = delta
        self.percent = self.value = 0
        self.start_time = datetime.now()
        self.first_message = first_message
        self.display_width = display_width

    def __call__(self, progress=None, message='...'):
        if self.first_message is not None:
            print(self.first_message)
            self.first_message = None
        if progress is None:
            self.value += self.delta
        else:
            self.value = progress
        percent = 100 * self.value // self.max_value
        if self.percent != percent:
            import sys
            from xoutil.datetime import strfdelta
            self.percent = percent
            helix = _HELIX[percent % len(_HELIX)]
            elapsed = self.start_time.now() - self.start_time
            total = type(elapsed)(seconds=elapsed.total_seconds() * 100 / self.percent)
            progress_line = '\r{helix} {percent}% - "{elapsed}" of about "{total}"'.format(
                         helix=helix,
                         percent=percent,
                         elapsed=strfdelta(elapsed),
                         total=strfdelta(total))
            max_width = self.display_width or self._get_terminal_width()
            progress_line += ('{message: >%d}' % (max_width - len(progress_line) - 1)).format(message=message)
            print(progress_line, end=('' if percent != 100 else '\n\r'))
            sys.stdout.flush()

    def _get_terminal_width(self, default=120):
        return default



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# ---------------------------------------------------------------------
# xoutil._values
# ---------------------------------------------------------------------
# Copyright (c) 2013-2015 Merchise Autrement
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-04-12

'''Fixed special values like Unset, Ignored, Required, etc...

All these values are False but are intended in places where None could be a
valid value.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)


class UnsetType(object):
    '''Instances are unique by name (singletons).

    Special instance `Unset` is automatically created and intended to be used
    as default value to be sure `None` is returned in scenarios it could be a
    valid value.

    For example::

        >>> class X(object):
        ...      attr = None

        >>> getattr(X(), 'attr', Unset) is Unset
        False

    '''
    __slots__ = tuple(str(slot) for slot in ('name', '__name__', 'help'))
    __instances__ = {}

    def __new__(cls, name, **kwargs):
        klass = kwargs.pop('__singleton__', None)
        help = kwargs.pop('help', None)
        if (not kwargs and (klass is UnsetType) and (cls is klass) and
                (__name__ == cls.__module__)):
            self = cls.__instances__.get(name)
            if self is None:
                self = super(UnsetType, cls).__new__(cls)
                self.__name__ = self.name = str(name)
                self.help = help
                cls.__instances__[name] = self
            return self
        else:
            raise TypeError("cannot create 'UnsetType' instances")

    def __nonzero__(self):
        return False

    __bool__ = __nonzero__

    def __repr__(self):
        return self.name

    __str__ = __repr__


Unset = UnsetType('Unset', __singleton__=UnsetType,
                  help='False value where None could be a valid value.')

Undefined = UnsetType('Undefined', __singleton__=UnsetType,
                      help=('False value for local scope use or where Unset '
                            'could be a valid value.'))

Ignored = UnsetType('Ignored', __singleton__=UnsetType,
                    help=("To be used in arguments that are currently "
                          "ignored cause they are being deprecated. The "
                          "only valid reason to use `ignored` is to signal "
                          "ignored arguments in method's/function's "
                          "signature."))



from __future__ import absolute_import as _
import warnings

warnings.warn('xoutil.six is deprecated, use six directly.', stacklevel=2)

from six import *
from six import moves

moves = sys.modules[__name__ + ".moves"] = moves



# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# xoutil.validators
# ---------------------------------------------------------------------
# Copyright (c) 2013-2015 Merchise Autrement and Contributors
# Copyright (c) 2011, 2012 Medardo Rodríguez
# All rights reserved.
#
# Author: Medardo Rodriguez
# Contributors: see CONTRIBUTORS and HISTORY file
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the LICENCE attached (see LICENCE file) in the distribution
# package.
#
# Created 2014-05-06

'''Some generic value validators and regular expressions and validation
functions for several identifiers.

'''


from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import)

from .identifiers import (is_valid_identifier,   # noqa
                          is_valid_full_identifier,
                          is_valid_public_identifier,
                          is_valid_slug)


def is_type(cls):
    '''Return a validator with the same name as the type given as argument
    `value`.

    :param cls: Class or type or tuple of several types.

    '''
    def inner(obj):
        '''Check is a value object is a valid instance of (%s).'''
        return isinstance(obj, cls)

    if isinstance(cls, tuple):
        name = None
        for t in cls:
            if name:
                name += str('_or_%s' % t.__name__)
            else:
                name = t.__name__
    else:
        name = cls.__name__
    inner.__name__ = name
    inner.__doc__ = inner.__doc__ % name
    return inner


def check(value, validator, msg=None):
    '''Check a `value` with a `validator`.

    Argument `validator` could be a callable, a type, or a tuple of types.

    Return True if the value is valid.

    Examples::

      >>> check(1, int)
      True

      >>> check(10, lambda x: x <= 100, 'must be less than or equal to 100')
      True

      >>> check(11/2, (int, float))
      True

    '''
    if isinstance(validator, (type, tuple)):
        checker = is_type(validator)
    else:
        checker = validator
    if checker(value):
        return True
    else:
        if not msg:
            # TODO: Use the name of validator with `inspect.getattr_static`
            # when `xoutil.future` is ready
            msg = 'Invalid value "%s" of type "%s"' % (value, type(value))
        raise ValueError(msg)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# ---------------------------------------------------------------------
# xoutil._meta3
# ---------------------------------------------------------------------
# Copyright (c) 2013-2015 Merchise Autrement and Contributors
# All rights reserved.
#
# This is free software; you can redistribute it and/or modify it under
# the terms of the LICENCE attached in the distribution package.
#
# Created on 2013-04-29

'''Implements the meta() function using the Py3k syntax.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        unicode_literals as _py3_unicode,
                        absolute_import as _py3_abs_imports)

from six import PY3
assert PY3, 'This module should be loaded in Py3k only'


def metaclass(meta, **kwargs):
    class base:
        pass

    metabase = meta.__base__
    if metabase is object:  # when meta is type
        metabase = type

    class inner_meta(metabase):
        @classmethod
        def __prepare__(cls, name, bases, **kwargs):
            prepare = getattr(meta, '__prepare__', None)
            if prepare:
                return prepare(name, bases, **kwargs)
            else:
                return dict()

        def __new__(cls, name, bases, attrs, **kw):
            if name != '__inner__':
                bases = tuple(b for b in bases if not issubclass(b, base))
                if not bases:
                    bases = (object,)
                return meta(name, bases, attrs)
            else:
                return type.__new__(cls, name, bases, attrs)

        def __init__(self, name, bases, attrs, **kw):
            pass

    from xoutil.types import new_class
    kwds = dict(kwargs, metaclass=inner_meta)
    return new_class('__inner__', (base, ), kwds=kwds)
